/***************************************************************************
       dslib-1.2.0.cpp  -  Example of the new features in DSLib v1.2.0
                             -------------------
    begin                : Tue Jun 13 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <dynamicsystem.h>
#include <blockvariable.h>
#include <function.h>
#include <functioninput.h>
#include <functionoutput.h>
#include <loop.h>
#include <summingpoint.h>
#include <gain.h>
#include <integrator.h>
#include <settingsharedmemory.h>
#include <fstream>
#include <comediinput.h>
#include <comedioutput.h>
#include <linux/comedi.h>

#define LOGGING_SAMPLING_RATE 0.1
#define INPUT_OFFSET 250 * 0x100000

int main(int argc, char *argv[])
{
	// Defining a dynamic system
	CDynamicSystem Regulator;
	bool bNoError = true;
	bNoError = Regulator.SetTime(10000000) && bNoError;
	bNoError = Regulator.SetTime(50000 * Regulator.Time()) && bNoError;
	
	CLoop* pLLoop = new CLoop();
	bNoError = pLLoop->SetName("L") && bNoError;
	bNoError = pLLoop->SetScheduling(CLoop::PERIOD) && bNoError;
	bNoError = pLLoop->SetPeriod(1000000000 * 0.01) && bNoError;
	bNoError = pLLoop->SetPriority(0) && bNoError;
	bNoError = Regulator.AddComponent(pLLoop) && bNoError;
	
	// Comedi input
	CComediInput * pComediInput = new CComediInput();
	bNoError = pComediInput->SetName("comedi_input") && bNoError;
	bNoError = pComediInput->SetComediMinor(0) && bNoError;
	bNoError = pComediInput->SetComediSubdevice(0) && bNoError;
	bNoError = pComediInput->SetComediChannel(7) && bNoError;
	bNoError = pComediInput->SetComediRange(0) && bNoError;
	bNoError = pComediInput->SetComediReference(AREF_GROUND) && bNoError;
	// Following two lines set the range of the block's output from -10 to +10
	bNoError = pComediInput->SetRange(20) && bNoError;
	bNoError = pComediInput->SetOffset(-10) && bNoError;
	bNoError = pLLoop->AddComponent(pComediInput) && bNoError;

	// System output logging Fifo
	CLoggingFifo* pLFSystemOutput = new CLoggingFifo();
	bNoError = pLFSystemOutput->SetNumber(0) && bNoError;
	bNoError = pLFSystemOutput->SetSamplingRate(LOGGING_SAMPLING_RATE) &&
		bNoError;
	bNoError = pLFSystemOutput->SetSize(sizeof(float) * 20) && bNoError;
	bNoError = pComediInput->Output()->AddComponent(pLFSystemOutput) && bNoError;

	// Reference
	CBlockVariable * pBVReference = new CBlockVariable();
	bNoError = pBVReference->SetName("reference") && bNoError;
	bNoError = pBVReference->SetType("float") && bNoError;
	bNoError = pBVReference->SetInitialization("reference = 0;") && bNoError;

	// Setting shared memory for reference
	CSettingSharedMemory* pSSMReference = new CSettingSharedMemory();
	bNoError = pSSMReference->SetOffset(INPUT_OFFSET);
	bNoError = pBVReference->AddComponent(pSSMReference);

	// System output
	CFunctionInput * pFISystemOutput = new CFunctionInput();
	bNoError = pFISystemOutput->SetName("system_output") && bNoError;
	bNoError = pFISystemOutput->SetType("float") && bNoError;
	bNoError = pFISystemOutput->Connect(pComediInput->Output()) && bNoError;

	// Summing Point
	CFunction* pFSummingPoint = new CFunction();
	bNoError = pFSummingPoint->SetName("summing_point") && bNoError;
	bNoError = pFSummingPoint->SetCode("error = reference - system_output;") &&
		bNoError;
	bNoError = pFSummingPoint->AddComponent(pBVReference) && bNoError;
	bNoError = pFSummingPoint->AddComponent(pFISystemOutput) && bNoError;
	bNoError = pLLoop->AddComponent(pFSummingPoint) && bNoError;

	// Error
	CFunctionOutput * pFOError = new CFunctionOutput();
	bNoError = pFOError->SetName("error") && bNoError;
	bNoError = pFOError->SetType("float") && bNoError;
	bNoError = pFSummingPoint->AddComponent(pFOError) && bNoError;

	// Error logging Fifo
	CLoggingFifo* pLFError = new CLoggingFifo();
	bNoError = pLFError->SetNumber(1) && bNoError;
	bNoError = pLFError->SetSamplingRate(LOGGING_SAMPLING_RATE) &&
		bNoError;
	bNoError = pLFError->SetSize(sizeof(float) * 20) && bNoError;
	bNoError = pFOError->AddComponent(pLFError) && bNoError;

	// Gain
	CGain* pGain = new CGain();
	bNoError = pGain->SetName("Gain") && bNoError;
	bNoError = pGain->SetConstantFactor(0.01) && bNoError;
	bNoError = pGain->Input()->Connect(pFOError) && bNoError;
	bNoError = pGain->SetIOsType("float") && bNoError;
	bNoError = pLLoop->AddComponent(pGain) && bNoError;
	
	// Integrator
	CIntegrator* pIntegrator = new CIntegrator();
	bNoError = pIntegrator->SetName("Integrator") && bNoError;
	bNoError = pIntegrator->Input()->Connect(pGain->Output()) && bNoError;
	bNoError = pIntegrator->SetIOsType("float") && bNoError;
	bNoError = pLLoop->AddComponent(pIntegrator) && bNoError;
	
	// Regulator output logging fifo
	CLoggingFifo* pLFRegulatorOutput = new CLoggingFifo();
	bNoError = pLFRegulatorOutput->SetNumber(2) && bNoError;
	bNoError = pLFRegulatorOutput->SetSamplingRate(LOGGING_SAMPLING_RATE) &&
		bNoError;
	bNoError = pLFRegulatorOutput->SetSize(sizeof(float) * 20) && bNoError;
	bNoError = pIntegrator->Output()->AddComponent(pLFRegulatorOutput) && bNoError;

	// Comedi Output
	CComediOutput * pComediOutput = new CComediOutput();
	bNoError = pComediOutput->SetName("ComediOutput") && bNoError;
	bNoError = pComediOutput->SetComediMinor(1) && bNoError;
	bNoError = pComediOutput->SetComediSubdevice(1) && bNoError;
	bNoError = pComediOutput->SetComediChannel(0) && bNoError;
	bNoError = pComediOutput->SetComediRange(0) && bNoError;
	bNoError = pComediOutput->SetComediReference(AREF_GROUND) && bNoError;
	// Following two lines set the range of the block's input from -20 to +20
	bNoError = pComediOutput->SetRange(40) && bNoError;
	bNoError = pComediOutput->SetOffset(-20) && bNoError;
	bNoError = pComediOutput->Input()->Connect(pIntegrator->Output());
	bNoError = pLLoop->AddComponent(pComediOutput) && bNoError;
	
	// Source code generation
	bNoError = Regulator.ArrangeAll() && bNoError;

	if (!bNoError)
	{
		CErrors Errors;
		Regulator.ReportErrors(Errors);
		CErrors::ConstIterator ErrorsIterator;
		for (ErrorsIterator = Errors.Begin(); ErrorsIterator != Errors.End();
			ErrorsIterator++)
		{
			cout << (*ErrorsIterator).Number() << endl;
			cout << (*ErrorsIterator).Message() << endl;
			cout << (*ErrorsIterator).Location()->Class() << endl;
			cout << (*ErrorsIterator).Location()->Identifier() << endl;
			cout << (*ErrorsIterator).Location()->Container()->Identifier() << endl;
		}
	}

	ofstream FileStream;
	FileStream.open("../rtmodule/comedirtmod.c");
	Regulator.GenerateSource(FileStream);
	FileStream.close();
	Regulator.DeleteObjects();

	return EXIT_SUCCESS;
}
